/*jslint browser: true*/
/*global define, google*/
define('de_epages/externalcontent/gadgetdialog/rss-view', [
    'jquery',
    './config-view',
    '$tmpl!./rss-view',
    '$dict!../dictionary',

    '../ui/rss' // The gadget to configure.
], function ($, ConfigView, template, dict) {
    'use strict';

    var RssView = ConfigView.extend({
        alias: 'rss',
        dict: dict,
        template: template,
        events: $.extend({}, ConfigView.events, {
            'keyup input[name="url"]': 'onUrlKeypress'
        }),
        getAPI: function (callback) {
            // Load Google Loader API.
            $.ajax({
                url: 'https://www.google.com/jsapi',
                dataType: 'script',
                cache: true // We would not want to load the same *script* twice.
            }).done(function () {
                // Load Google Feed API.
                google.load('feeds', '1', {
                    callback: callback || function () {}
                });
            });
        },
        getFeed: function (url) {
            this.getAPI($.proxy(function () {
                var self = this,
                    feed = new google.feeds.Feed(url);
                self.elements.verifyingUrl.removeClass('fail-icon success-icon').addClass('wait-icon');
                feed.load(function (result) {
                    if (result.error) {
                        self.elements.verifyingUrl.removeClass('wait-icon success-icon').addClass('fail-icon');
                    } else {
                        self.elements.verifyingUrl.removeClass('wait-icon fail-icon').addClass('success-icon');
                        self.setOption('url', url, true);
                    }
                });
            }, this));
        },
//#JSCOVERAGE_IF 0
        onUrlKeypress: function (e) {
            var self = this;
            clearTimeout(self.verifyUrlTimeout);
            // Wait for user to type in (hopefully valid) feed url.
            self.verifyUrlTimeout = setTimeout(function () {
                self.getFeed(e.target.value);
            }, 500);
        }
//#JSCOVERAGE_ENDIF
    });
    return RssView;
});