/*jslint nomen: true*/
/*global define*/

define("de_epages/mediagallery/filemanager/navigation-view", [
	"backbone",
	"underscore",
	"jquery",
	"$tmpl!./navigation-view",
	"./tree-view",
	"$dict!../dictionary"
], function (Backbone, _, $, template, TreeView, mediagalleryDict) {
    "use strict";

    var tFolder = mediagalleryDict.translate("Folder"),

        view = Backbone.View.extend({
            tagName: "div",
            className: "ep-mediagallery-navigation",
            template: template,

            initialize: function (options) {
                this.options = options;

                this
	                // trigger events to single tree views
	                .on("addTreeNode", this.addTreeNode, this).on("renameTreeNode", this.renameTreeNode, this).on("removeTreeNode", this.removeTreeNode, this);

                this.treeViews = [];

                // create tree view for each main folder
                this.collection.each(function (item, i) {
                    this.treeViews[i] = new TreeView({
                        model: item,
                        treeData: this.options.rootFolderData[i],
                        currentFolder: this.options.currentFolder
                    });
                    this.treeViews[i].on("openFolder", this.triggerOpenFolder, this).on("changeState", this.changeState, this).on("folderEditable", this.triggerFolderEditable, this);
                }, this);
            },

            render: function () {
                var tmpl = this.template({
                    tFolder: tFolder
                });

                this.$el.append(tmpl);

                $.extend(this, $.tmplItem(tmpl).elements);

                _.each(this.treeViews, function (item) {
                    this.treeList.append(item.render().el);
                }, this);

                return this;
            },

            getOpenFolder: function () {
                return this.collection.where({ state: "open" })[0];
            },

            addTreeNode: function (data) {
                var model = this.getOpenFolder();
                model.trigger("addTreeNode", data);
            },

            renameTreeNode: function (data) {
                var model = this.getOpenFolder();
                model.trigger("renameTreeNode", data);
            },

            removeTreeNode: function (data) {
                var model = this.getOpenFolder();
                model.trigger("removeTreeNode", data);
            },

            setActiveNode: function (id) {
                var model = this.getOpenFolder();
                model.trigger("setActiveNode", id);
            },

            triggerOpenFolder: function (data) {
                this.trigger("openFolder", data);
            },

            triggerFolderEditable: function (editable) {
                this.trigger("folderEditable", editable);
            },

            // change state of an opened tree to closed
            changeState: function () {
                var model = this.getOpenFolder();
                model.set("state", "closed");
            }
        });

    return view;
});