/*globals define*/
/*jslint nomen:true, regexp:true*/

define([
    'testsuite/qunit',
    'jquery',
    'ep',
    '$dict!../dictionary',
    './content-view',
    './tray-collection',
    'ep/ui/datatable',
    'ep/ajax',
    '$ready!'
], function (QUnit, $, ep, dict, ContentGeneralView, TrayCollection) {
    'use strict';

    var fixture = $('#qunit-fixture'),
        view,
        dataTable,
        initSearch,
        initNormal,
        initAdditionalContent,
        data = {
            "tableHead": [{
                "isSortable": 0,
                "name": "",
                "id": "ObjectID",
                "isSelectable": 1
            }, {
                "isSortable": 1,
                "name": "Product number",
                "id": "Alias",
                "isSelectable": 0
            }, {
                "isSortable": 1,
                "name": "",
                "id": "NameOrAlias",
                "isLinkable": 1,
                "isSelectable": 0
            }, {
                "isSortable": 0,
                "name": "",
                "id": "VariationAttr",
                "isSelectable": 0
            }, {
                "isSortable": 1,
                "name": "List price",
                "id": "ListPrice",
                "isSelectable": 0
            }],
            "totalNumber": "22",
            "orderBy": "NameOrAlias",
            "lastPage": 3,
            "orderDesc": 0,
            "tableContent": {
                "1": [{
                    "Alias": "be_40401",
                    "Parents": [],
                    "ObjectID": 17144,
                    "VariationAttr": null,
                    "NameOrAlias": "Berghaus Paclite Jacket - Men",
                    "hasSubTable": 1,
                    "ListPrice": "199.95 €"
                }, {
                    "Alias": "be_40402",
                    "Parents": [],
                    "ObjectID": 17156,
                    "VariationAttr": null,
                    "NameOrAlias": "Berghaus Paclite Jacket - Women",
                    "hasSubTable": 1,
                    "ListPrice": "199.95 €"
                }, {
                    "Alias": "vs_3202112018",
                    "Parents": [],
                    "ObjectID": 17215,
                    "VariationAttr": null,
                    "NameOrAlias": "Black Bear Gemini",
                    "ListPrice": "29.95 €"
                }, {
                    "Alias": "cg_0101004270",
                    "Parents": [],
                    "ObjectID": 17137,
                    "VariationAttr": null,
                    "NameOrAlias": "Camping AZ CV270 valve gas canister",
                    "ListPrice": "3.95 €"
                }, {
                    "Alias": "cg_0101104470",
                    "Parents": [],
                    "ObjectID": 17138,
                    "VariationAttr": null,
                    "NameOrAlias": "Camping AZ CV470 valve Gas canister",
                    "ListPrice": "7.95 €"
                }, {
                    "Alias": "cg_0100504001",
                    "Parents": [],
                    "ObjectID": 17136,
                    "VariationAttr": null,
                    "NameOrAlias": "Campingaz Twister 270",
                    "ListPrice": "22.95 €"
                }, {
                    "Alias": "de_3201212002",
                    "Parents": [],
                    "ObjectID": 17212,
                    "VariationAttr": null,
                    "NameOrAlias": "Deuter Hydro 2.0",
                    "ListPrice": "74.95 €"
                }, {
                    "Alias": "de_3203104010",
                    "Parents": [],
                    "ObjectID": 17213,
                    "VariationAttr": null,
                    "NameOrAlias": "Deuter Kangaroo",
                    "ListPrice": "99.95 €"
                }, {
                    "Alias": "de_3206199010",
                    "Parents": [],
                    "ObjectID": 17216,
                    "VariationAttr": null,
                    "NameOrAlias": "Deuter Teddy Bear",
                    "ListPrice": "26.95 €"
                }, {
                    "Alias": "er_7142303001",
                    "Parents": [],
                    "ObjectID": 17139,
                    "VariationAttr": null,
                    "NameOrAlias": "Edelrid Black Bear Rope",
                    "ListPrice": "1.45 €"
                }],
                "2": [{
                    "Alias": "be_40401",
                    "Parents": [],
                    "ObjectID": 17144,
                    "VariationAttr": null,
                    "NameOrAlias": "Berghaus Paclite Jacket - Men",
                    "hasSubTable": 1,
                    "ListPrice": "199.95 €"
                }, {
                    "Alias": "be_40402",
                    "Parents": [],
                    "ObjectID": 17156,
                    "VariationAttr": null,
                    "NameOrAlias": "Berghaus Paclite Jacket - Women",
                    "hasSubTable": 1,
                    "ListPrice": "199.95 €"
                }]
            }
        },
        _ajax = ep.ajax;

    ep.ajax = function (o) {
        if (o.data) {
            if (o.data.ViewAction === 'JSONSearchProductsToInsert') {
                return {
                    done: function (f) {
                        f(data);
                    }
                };
            }
        } else {
            return _ajax(o);
        }
    };

    initSearch = function () {
        view = new ContentGeneralView(initSearch.options);

        view.render().$el.appendTo(fixture);
        dataTable = view.$('table');

        return view;
    };

    initSearch.options = {
        context: 'products',
        searchAction: 'JSONSearchProductsToInsert',
        searchString: 'TestSearchString',
        collection: new TrayCollection(),
        additionalContent: undefined
    };

    initNormal = function () {
        view = new ContentGeneralView(initNormal.options);

        view.render().$el.appendTo(fixture);
        dataTable = view.$('table');

        return view;
    };

    initNormal.options = {
        context: 'products',
        searchAction: 'JSONSearchProductsToInsert',
        collection: new TrayCollection(),
        additionalContent: undefined
    };

    initAdditionalContent = function () {
        view = new ContentGeneralView(initAdditionalContent.options);

        view.render().$el.appendTo(fixture);
        dataTable = view.$('table');

        return view;
    };

    initAdditionalContent.options = {
        context: 'products',
        searchAction: 'JSONSearchProductsToInsert',
        collection: new TrayCollection(),
        additionalContent: '<input id="ep-tray-categories-additionalcontent" name="myAddContent" type="checkbox" />'
    };

    QUnit.module("de_epages/presentation/tray/content-general-view", {
        setup: function () {},
        teardown: function () {
            fixture.empty();
        }
    });

    QUnit.test('Check content-general-view\'s public methods', 2, function () {
        initNormal();

        $.each(['initialize', 'render'], function () {
            QUnit.ok(
                $.isFunction(view[this]),
                'Check if ' + this + ' is a method of the content-general-view'
            );
        });
    });

    QUnit.test('Check content-general-view\'s stuff', 2, function () {
        initSearch();

        QUnit.deepEqual(
            view.options,
            initSearch.options,
            'Options are set correctly'
        );

        QUnit.strictEqual(
            view.searchField.val(),
            initSearch.options.searchString,
            'Search string is set correctly'
        );
    });

    QUnit.test('Checking search field (with initial search string)', 9, function () {
        initSearch();

        QUnit.strictEqual(
            view.resetButton.hasClass('Opacity0'),
            false,
            'Check if reset button is initially there'
        );

        view.searchField.val('test').trigger('keyup');
        QUnit.strictEqual(
            view.resetButton.hasClass('Opacity0'),
            false,
            'Check if reset button is still there after typing'
        );

        view.searchField.val('').trigger('keyup');
        QUnit.ok(
            view.resetButton.hasClass('Opacity0'),
            'Check if reset button is hidden after search string is cleared by user'
        );

        view.on('search', function (searchString) {
            QUnit.strictEqual(
                searchString,
                'Another Test',
                'Check if the search event is triggered with the correct search string on click'
            );
        });

        view.searchField.val('Another Test').trigger('keyup');
        view.searchButton.trigger('click');

        view.off('search');

        view.once('search', function (searchString) {
            QUnit.strictEqual(
                searchString,
                'My Third Test',
                'Check if the search event is triggered with the correct search string on enter'
            );
        });

        view.searchField.val('My Third Test').trigger('keyup');
        var e = $.Event("keyup");
        e.which = 13;
        view.searchField.trigger(e);

        view.resetButton.trigger('click');
        QUnit.strictEqual(
            view.searchField.val(),
            '',
            'Check if click on the reset button clears the search field'
        );

        $(view.$('tbody input')[0]).trigger('click');
        QUnit.strictEqual(
            view.collection.length,
            1,
            'Check if element is added to collection'
        );

        $(view.$('tfoot .next')[0]).trigger('click');
        $(view.$('tfoot .back')[1]).trigger('click');
        QUnit.ok(
            $(view.$('tbody input')[0]).is(':checked'),
            'Check if input is re-checked after switching the page'
        );

        $(view.$('tbody input')[0]).trigger('click');
        QUnit.strictEqual(
            view.collection.length,
            0,
            'Check if element is removed from collection'
        );
    });

    QUnit.test('Check search field (without initial search string)', function () {
        initNormal();

        QUnit.ok(
            view.resetButton.hasClass('Opacity0'),
            'Check if reset button is initially hidden'
        );

        view.searchField.val('test').trigger('keyup');
        QUnit.strictEqual(
            view.resetButton.hasClass('Opacity0'),
            false,
            'Check if reset button is revealed after typing'
        );

        view.searchField.val('').trigger('keyup');
        QUnit.ok(
            view.resetButton.hasClass('Opacity0'),
            'Check if reset button is hidden after search string is cleared by user'
        );
    });

    QUnit.test('Check changecontent event', function () {
        initNormal();

        var triggered = false,
            id,
            context;

        view.on('changecontent', function (eventId, eventContext) {
            triggered = true;

            id = eventId;
            context = eventContext;
        });

        fixture.find('a[data-id=17144]').first().trigger('click');

        QUnit.ok(triggered);

        QUnit.strictEqual(
            id,
            17144,
            'Check if triggered event contains the correct ID'
        );

        QUnit.strictEqual(
            context,
            'products',
            'Check if triggered event contains the correct context'
        );
    });

    QUnit.test('Check path for search results', function () {
        initSearch();

        view.table.uiDatatable().trigger('datatablerendered', {
            "path": [{
                "ObjectID": "17017",
                "NameOrAlias": "Products"
            }, {
                "ObjectID": "17144",
                "NameOrAlias": "Berghaus Paclite Jacket - Men"
            }]
        });

        QUnit.equal(
            fixture.find('div > div > div').first().text(),
            dict.translate('SearchResults'),
            'Check if path is set to ' + dict.translate('SearchResults')
        );
    });

    QUnit.test('Check path for normal results', function () {
        initNormal();

        var pathDiv = fixture.find('div > div > div').first(),
            id;

        view.table.uiDatatable().trigger('datatablerendered', {
            "path": [{
                "ObjectID": "17017",
                "NameOrAlias": "Products"
            }, {
                "ObjectID": "17144",
                "NameOrAlias": "Berghaus Paclite Jacket - Men"
            }]
        });

        /(.*Products.*Berghaus Paclite Jacket - Men)/.exec(pathDiv.text());
        QUnit.equal(
            pathDiv.text(),
            RegExp.$1,
            'Check if path is set to "Products > Berghaus Paclite Jacket - Men"'
        );

        /* TRIGGER CLICK ON "Products" TO START NAVIGATE */

        view.on('changecontent', function (eventId) {
            id = eventId;
        });

        pathDiv.find('a[data-id=17017]').trigger('click');

        QUnit.strictEqual(
            id,
            17017,
            'Check if a click on "Products" triggers the changecontent event with 17017 as ID'
        );
    });

    QUnit.test('Check if additional content is rendered and returned', function () {
        var view, additionalContent, input;
        view = initAdditionalContent();

        additionalContent = fixture.find('.additional-content');
        QUnit.strictEqual(
            additionalContent.length,
            1,
            'Check is there is a place for the additional content'
        );

        input = additionalContent.find('input[type=checkbox]#ep-tray-categories-additionalcontent');
        QUnit.strictEqual(
            input.length,
            1,
            'Check is the input is a child of the additional content element'
        );

        QUnit.deepEqual(view.getAdditionalContent(), {
            'myAddContent': false
        }, 'Check if the *myAddContent* attribute is initially false');

        input.trigger('click');

        QUnit.deepEqual(view.getAdditionalContent(), {
            'myAddContent': true
        }, 'Check if the *myAddContent* attribute is true after a click on the checkbox');
    });
});